@@ -10,6 +10,7 @@ from box import Box
 # Related modules
 from .. import common
 from .. import data
+from .. import utils
 from ..data.validate import must_be_string,must_be_list,must_be_dict,validate_attributes
 from .. import addressing
 from . import devices
@@ -79,7 +80,7 @@ def adjust_link_list(links: list, nodes: Box) -> list:
       link_list.append(link_data)                               # ... and move on
     elif isinstance(l,list):
       link_list.append(Box({ 'interfaces': adjust_interface_list(l,l,nodes) },default_box=True,box_dots=True))
-    else:                                   # Assuming the link value is a string, split
+    elif isinstance(l,str):
       link_intf = []
       for n in l.split('-'):                # ... split it into a list of nodes
         if n in nodes:                      # If the node name is valid
@@ -90,12 +91,19 @@ def adjust_link_list(links: list, nodes: Box) -> list:
             common.IncorrectValue,
             'links')
       link_list.append({ 'interfaces': link_intf })
+    else:                                   # Come on, you really should not do something like this
+      common.error(
+        f'Invalid type {type(l).__name__} for a link instance: {l}',
+        common.IncorrectType,
+        'links')
+      continue
+
     link_cnt = link_cnt + 1
 
   if common.debug_active('links'):
     print("Adjusted link list")
     print("=" * 60)
-    print(common.print_yaml(link_list))
+    print(common.get_yaml_string(link_list))
 
   return link_list
 
@@ -103,6 +111,7 @@ def adjust_link_list(links: list, nodes: Box) -> list:
 Validate link attributes
 """
 def validate(topology: Box) -> None:
+  providers = list(topology.defaults.providers.keys())
   for l_data in topology.links:
     linkpath = f'links[{l_data.linkindex}]'           # Topology path to current link
     validate_attributes(
@@ -112,6 +121,7 @@ def validate(topology: Box) -> None:
       data_name=f'link',
       attr_list=['link'],                             # We're checking node attributes
       modules=topology.get('module',[]),              # ... against topology modules
+      extra_attributes=providers,                     # Allow provider-specific attributes in links
       module_source='topology',
       module='links')                                 # Function is called from 'links' module
 
@@ -155,29 +165,61 @@ Add interface data structure to a node:
 * Cleanup 'af: False' entries
 * Handle interface/node/system MTU
 """
-def add_node_interface(node: Box, ifdata: Box, defaults: Box) -> Box:
+def create_regular_interface(node: Box, ifdata: Box, defaults: Box) -> None:
   ifindex_offset = devices.get_device_attribute(node,'ifindex_offset',defaults)
   if ifindex_offset is None:
     ifindex_offset = 1
 
   # Allow user to select a specific interface index per link
-  ifindex = ifdata.get('ifindex',None) or (len(node.interfaces) + ifindex_offset)
+  ifindex = ifdata.get('ifindex',None)
+
+  # When computing default ifindex, consider only non-loopback interfaces
+  if not ifindex:
+   ifindex = len([intf for intf in node.interfaces if intf.get('type',None) != 'loopback']) + ifindex_offset
 
   ifname_format = devices.get_device_attribute(node,'interface_name',defaults)
 
   ifdata.ifindex = ifindex
   if ifname_format and not 'ifname' in ifdata:
-    ifdata.ifname = ifname_format % ifindex
+    ifdata.ifname = utils.strings.eval_format(ifname_format,ifdata)
 
   pdata = devices.get_provider_data(node,defaults).get('interface',{})
   pdata = Box(pdata,box_dots=True,default_box=True)                     # Create a copy of the provider interface data
   if 'name' in pdata:
-    pdata.name = pdata.name % ifindex
+    pdata.name = utils.strings.eval_format(pdata.name,ifdata)
 
   if pdata:
     provider = devices.get_provider(node,defaults)
     ifdata[provider] = pdata
 
+def create_loopback_interface(node: Box, ifdata: Box, defaults: Box) -> None:
+  ifindex_offset = devices.get_device_attribute(node,'loopback_offset',defaults) or 0
+  ifdata.ifindex = ifindex_offset + ifdata.linkindex
+  ifdata.virtual_interface = True
+  ifname_format  = devices.get_device_attribute(node,'loopback_interface_name',defaults)
+
+  if not ifname_format:
+    common.error(
+      f'Device {node.device}/node {node.name} does not support loopback links',
+      common.IncorrectValue,
+      'links')
+    return
+
+  if not 'ifname' in ifdata:
+    ifdata.ifname = utils.strings.eval_format(ifname_format,ifdata)
+
+  # If the device uses 'loopback_offset' then we're assuming it's large enough to prevent overlap with physical interfaces
+  # Otherwise, create fake ifindex for loopback interfaces to prevent that overlap
+  #
+  if not ifindex_offset:
+    ifdata.ifindex = 10000 + ifdata.ifindex
+
+def add_node_interface(node: Box, ifdata: Box, defaults: Box) -> Box:
+  if ifdata.get('type') == 'loopback':
+    create_loopback_interface(node,ifdata,defaults)
+  else:
+    create_regular_interface(node,ifdata,defaults)
+
   for af in ('ipv4','ipv6'):
     if af in ifdata and not ifdata[af]:
       del ifdata[af]
@@ -215,7 +257,7 @@ def interface_data(link: Box, link_attr: set, ifdata: Box) -> Box:
 Set FHRP (anycast/VRRP/...) gateway on a link
 """
 
-def set_fhrp_gateway(link: Box, pfx_list: Box, link_path: str) -> None:
+def set_fhrp_gateway(link: Box, pfx_list: Box, nodes: Box, link_path: str) -> None:
   gwid = data.get_from_box(link,'gateway.id')
   if not gwid:                                                        # No usable gateway ID, nothing to do
     return
@@ -223,9 +265,9 @@ def set_fhrp_gateway(link: Box, pfx_list: Box, link_path: str) -> None:
   fhrp_assigned = False
   for af in common.AF_LIST:
     if not af in pfx_list or isinstance(pfx_list[af],bool):           # No usable IPv4/IPv6 prefix, nothing to do
-      return
+      continue
 
-    try:                                                                # Now try to get N-th IP address on that link
+    try:                                                              # Now try to get N-th IP address on that link
       link.gateway[af] = get_nth_ip_from_prefix(netaddr.IPNetwork(link.prefix[af]),link.gateway.id)
       fhrp_assigned = True
     except Exception as ex:
@@ -237,8 +279,17 @@ def set_fhrp_gateway(link: Box, pfx_list: Box, link_path: str) -> None:
         'links')
       return
 
-  if common.debug_active('links') and fhrp_assigned:
-    print(f'... Assigning FHRP address(es): {link.gateway}')
+  if not fhrp_assigned:
+    return
+
+  for intf in link.interfaces:                                        # Copy link gateway into interface attributes
+    if 'gateway' in nodes[intf.node].get('module',[]):                # ... but only for nodes using the gateway module
+      for af in common.AF_LIST:
+        if af in link.gateway:
+          intf.gateway[af] = link.gateway[af]
+
+  if common.debug_active('links'):     # pragma: no cover (debugging)
+    print(f'FHRP gateway set for {link}')
 
 """
 Assign a prefix (IPv4+IPv6) to a link:
@@ -254,13 +305,18 @@ Allocating pool prefix:
 * Allocate a prefix from the first available candidate pool
 """
 
-def assign_link_prefix(link: Box,pools: typing.List[str],addr_pools: Box,link_path: str = 'links') -> Box:
+def assign_link_prefix(
+      link: Box,
+      pools: typing.List[str],
+      addr_pools: Box,
+      nodes: Box,
+      link_path: str = 'links') -> Box:
   if 'prefix' in link:                                    # User specified a static link prefix
     pfx_list = addressing.parse_prefix(link.prefix)
     if isinstance(link.prefix,str):
       link.prefix = addressing.rebuild_prefix(pfx_list)   # convert str to prefix dictionary
 
-    set_fhrp_gateway(link,pfx_list,link_path)
+    set_fhrp_gateway(link,pfx_list,nodes,link_path)
     return pfx_list
 
   if 'unnumbered' in link:                                # User requested an unnumbered link
@@ -285,7 +341,7 @@ def assign_link_prefix(link: Box,pools: typing.List[str],addr_pools: Box,link_pa
   if not link.prefix:
     link.pop('prefix',None)
 
-  set_fhrp_gateway(link,pfx_list,link_path)
+  set_fhrp_gateway(link,pfx_list,nodes,link_path)
   return pfx_list
 
 """
@@ -319,10 +375,13 @@ def get_prefix_IPAM_policy(link: Box, pfx: typing.Union[netaddr.IPNetwork,bool],
     pfx_size = pfx_size + subtract_reserved_ip
 
   max_id = max([ ndict[intf.node].id for intf in link.interfaces if intf.node in ndict ])
-  if max_id < pfx_size:
-    return 'id_based'
-  if len(link.interfaces) + add_extra_ip < pfx_size:
-    return 'sequential'
+  if max_id < pfx_size:                                               # If we can fit all node IDs attached to this link into the prefix
+    return 'id_based'                                                 # ... we'll use ID-based address allocation
+  if len(link.interfaces) + add_extra_ip < pfx_size:                  # Otherwise, if the prefix is big enough
+    return 'sequential'                                               # ... we'll use sequential address allocation
+
+  if pfx_size < 2 and link.type == 'loopback':                        # Final straw: maybe we're dealing with a loopback?
+    return 'loopback'
 
   return 'error'
 
@@ -419,11 +478,17 @@ def IPAM_id_based(link: Box, af: str, pfx: netaddr.IPNetwork, ndict: Box) -> Non
   for intf in link.interfaces:
     set_interface_address(intf,af,pfx,ndict[intf.node].id)
 
+def IPAM_loopback(link: Box, af: str, pfx: netaddr.IPNetwork, ndict: Box) -> None:
+  for intf in link.interfaces:
+    pfx.prefixlen = 128 if ':' in str(pfx) else 32
+    intf[af] = str(pfx)
+
 IPAM_dispatch: typing.Final[dict] = { 
     'unnumbered': IPAM_unnumbered,
     'p2p': IPAM_p2p,
     'sequential': IPAM_sequential,
-    'id_based': IPAM_id_based
+    'id_based': IPAM_id_based,
+    'loopback': IPAM_loopback
   }
 
 """
@@ -477,7 +542,8 @@ def assign_interface_addresses(link: Box, addr_pools: Box, ndict: Box, defaults:
       if data.get_from_box(link,'gateway.id'):
         rq = rq + f' plus first-hop gateway'
       common.error(
-        f'Cannot use {af} prefix {pfx_list[af]} to address {rq} on link#{link.linkindex}',
+        f'Cannot use {af} prefix {pfx_list[af]} to address {rq} on link#{link.linkindex}\n' + \
+        common.extra_data_printout(f'link data: {link}',width=90),
         common.IncorrectValue,
         'links')
       continue
@@ -548,7 +614,7 @@ def set_interface_name(ifdata: Box, link: Box, ifcnt: int) -> None:
 """
 Create node interfaces from link interfaces
 """
-def create_node_interfaces(link: Box, addr_pools: Box, ndict: dict, defaults: Box) -> None:
+def create_node_interfaces(link: Box, addr_pools: Box, ndict: Box, defaults: Box) -> None:
   link_attr_propagate = get_link_propagate_attributes(defaults)
 
   if common.debug_active('links'):     # pragma: no cover (debugging)
@@ -573,7 +639,8 @@ def create_node_interfaces(link: Box, addr_pools: Box, ndict: dict, defaults: Bo
                                                                   # ... must use dict not Box as Box creates a copy of the data structure
 
   if common.debug_active('links'):     # pragma: no cover (debugging)
-    print(f'... LAN link data: {link}\n')
+    print(f'... link data: {link}')
+    print(f'... interface data: {interfaces}\n')
 
   # Second phase: build neighbor list from list of newly-created interfaces
   for node_if in interfaces:
@@ -588,7 +655,8 @@ def create_node_interfaces(link: Box, addr_pools: Box, ndict: dict, defaults: Bo
       #
       # Find relevant modules that have interface attributes
       mods_with_attr = set([ m for m in ndict[remote_node].get('module',[])
-                              if defaults[m].attributes.get('interface',None) ])
+                              if defaults[m].attributes.get('interface',None) or
+                                 defaults[m].attributes.get('link_to_neighbor',None) ])
       #
       # Merge neighbor module data + AF with baseline neighbor data
       ngh_data = interface_data(
@@ -597,7 +665,24 @@ def create_node_interfaces(link: Box, addr_pools: Box, ndict: dict, defaults: Bo
                    ifdata=ngh_data)
       ifdata.neighbors.append(ngh_data)
 
-def set_link_type_role(link: Box, pools: Box, nodes: Box) -> None:
+def set_link_loopback_type(link: Box, nodes: Box, defaults: Box) -> None:
+  node = link.interfaces[0].node
+  ndata = nodes[node]
+  features = devices.get_device_features(ndata,defaults)
+
+  lb_name = devices.get_device_attribute(ndata,'loopback_interface_name',defaults)
+  if not lb_name:
+    return
+
+  if features.stub_loopback is False:
+    return
+
+  if not defaults.links.stub_loopback and not features.stub_loopback:
+    return
+
+  link.type = 'loopback'
+
+def set_link_type_role(link: Box, pools: Box, nodes: Box, defaults: Box) -> None:
   node_cnt = len(link.interfaces)   # Set the number of attached nodes (used in many places further on)
   link['node_count'] = node_cnt
 
@@ -615,6 +700,8 @@ def set_link_type_role(link: Box, pools: Box, nodes: Box) -> None:
     return
 
   link.type = 'lan' if node_cnt > 2 else 'p2p' if node_cnt == 2 else 'stub'     # Set link type based on number of attached nodes
+  if node_cnt == 1:
+    set_link_loopback_type(link,nodes,defaults)
 
   if host_count > 0:
     link.type = 'lan'
@@ -655,8 +742,18 @@ def check_link_type(data: Box) -> bool:
     common.error('Point-to-point link needs exactly two nodes: %s' % data,common.IncorrectValue,'links')
     return False
 
-  if not link_type in [ 'stub','p2p','lan','vlan_member']:
-    common.error('Invalid link type %s: %s' % (link_type,data),common.IncorrectValue,'links')
+  if link_type == 'loopback' and node_cnt != 1:
+    common.error(
+      f'Looopback link can have a single node attached: links[{data.linkindex}]\n... {data}',
+      common.IncorrectValue,
+      'links')
+    return False
+
+  if not link_type in [ 'stub','p2p','lan','loopback','vlan_member']:
+    common.error(
+      f'Invalid link type {link_type} in links[{data.linkindex}]\n... {data}',
+      common.IncorrectValue,
+      'links')
     return False
   return True
 
@@ -688,12 +785,15 @@ def interface_feature_check(nodes: Box, defaults: Box) -> None:
 def set_default_gateway(link: Box, nodes: Box) -> None:
   if not 'host_count' in link:      # No hosts attached to the link, get out
     return
+  link.pop('host_count',None)
 
   # No IPv4 prefix on the link or unnumbered IPv4 link
   if not 'ipv4' in link.prefix or isinstance(link.prefix.ipv4,bool):
     return
 
-  link.pop('host_count',None)
+#  if 'vlan_name' in link:                                 # Do not try to set first-hop gateways on VLAN links, VLAN module will do that
+#    return
+
   if common.debug_active('links'):
     print(f'Set DGW for {link}')
   if not 'gateway' in link:
@@ -705,7 +805,7 @@ def set_default_gateway(link: Box, nodes: Box) -> None:
   elif link.gateway is False:
     return
 
-  if not 'gateway' in link or not 'ipv4' in link.gateway: # Didn't find a usable gateway, exit
+  if not 'gateway' in link or not isinstance(link.gateway,Box) or not 'ipv4' in link.gateway: # Didn't find a usable gateway, exit
     if common.debug_active('links'):
       print('... not found')
     return
@@ -749,13 +849,13 @@ def transform(link_list: typing.Optional[Box], defaults: Box, nodes: Box, pools:
     return None
 
   for link in link_list:
-    set_link_type_role(link=link,pools=pools,nodes=nodes)
+    set_link_type_role(link=link,pools=pools,nodes=nodes,defaults=defaults)
     if not check_link_type(data=link):
       continue
 
     set_link_bridge_name(link,defaults)
     link_default_pools = ['p2p','lan'] if link.type == 'p2p' else ['lan']
-    assign_link_prefix(link,link_default_pools,pools,f'links[{link.linkindex}]')
+    assign_link_prefix(link,link_default_pools,pools,nodes,f'links[{link.linkindex}]')
     assign_interface_addresses(link,pools,nodes,defaults)
     create_node_interfaces(link,pools,nodes,defaults=defaults)
 
