@@ -9,28 +9,30 @@ from box import Box
 from .. import common
 from .. import addressing
 from .. import augment
-from ..providers import _Provider
+from .. import providers
 from .. import modules
-from .. import quirks
+from .. import devices as quirks
 from ..data import global_vars
 
 def transform_setup(topology: Box) -> None:
   global_vars.init(topology)
+  augment.config.attributes(topology)
   augment.topology.check_required_elements(topology)
   topology.nodes = augment.nodes.create_node_dict(topology.nodes)
   if 'links' in topology:
     topology.links = augment.links.adjust_link_list(topology.links,topology.nodes)
     augment.links.set_linkindex(topology)
   augment.devices.augment_device_settings(topology)
+  augment.plugin.init(topology)                                         # Initialize plugins very early on in case they modify extra attributes
+  augment.plugin.execute('init',topology)
+  common.exit_on_error()
+
+  augment.topology.extend_attribute_list(topology.defaults)             # Attributes have to be extended before group init
+  augment.topology.extend_module_attribute_list(topology)               # ... or we won't recognize node attributes in groups
   augment.groups.init_groups(topology)
   common.exit_on_error()
 
-  augment.plugin.init(topology)
-  augment.plugin.execute('init',topology)
-  augment.topology.extend_attribute_list(topology.defaults)
-  augment.topology.extend_module_attribute_list(topology)
   augment.topology.adjust_global_parameters(topology)
-  topology.Provider = _Provider.load(topology.provider,topology.defaults.providers[topology.provider])
   common.exit_on_error()
 
   augment.nodes.augment_node_provider_data(topology)
@@ -42,11 +44,14 @@ def transform_setup(topology: Box) -> None:
 
   augment.topology.check_global_elements(topology)
   augment.nodes.validate(topology)
+  common.exit_on_error()
 
 def transform_data(topology: Box) -> None:
   addressing.setup(topology)
+  common.exit_on_error()
   augment.plugin.execute('pre_transform',topology)
   modules.pre_transform(topology)
+  providers.execute("pre_transform",topology)
 
   augment.plugin.execute('pre_node_transform',topology)
   modules.pre_node_transform(topology)
@@ -70,15 +75,14 @@ def transform_data(topology: Box) -> None:
   modules.post_transform(topology)
   augment.plugin.execute('post_transform',topology)
   augment.groups.node_config_templates(topology)
-  topology.Provider.call("post_transform",topology)
+  providers.execute("post_transform",topology)
   common.exit_on_error()
 
   quirks.process_quirks(topology)
   common.exit_on_error()
   
-  topology.pop('Plugin',None)
-  del topology.pools
-  del topology.Provider
+  for remove_attr in ['Plugin','pools','_Providers']:
+    topology.pop(remove_attr,None)
 
 def transform(topology: Box) -> None:
   transform_setup(topology)
