@@ -63,14 +63,7 @@ curl -fsSL https://apt.releases.hashicorp.com/gpg | $SUDO gpg --dearmor -o /etc/
 $SUDO sh -c 'echo "deb [arch=amd64] https://apt.releases.hashicorp.com $(lsb_release -cs) main" > /etc/apt/sources.list.d/vagrant.list'
 $SUDO apt-get update
 $SUDO apt-get install -y $FLAG_QUIET ruby-dev ruby-libvirt vagrant
-vagrant plugin install vagrant-libvirt --plugin-version=0.4.1
-#
-# Temporary fix: replace fog-libvirt 0.10.1 with fog-libvirt 0.9.0 until we can upgrade to the latest vagrant-libvirt plugin
-#
-echo ".. replacing fog-libvirt gem with an older version"
-GEMS_DIR=`echo ~/.vagrant.d/gems/*`
-GEM_HOME=$GEMS_DIR gem uninstall -aIx fog-libvirt
-GEM_HOME=$GEMS_DIR gem install 'fog-libvirt:0.9.0'
+vagrant plugin install vagrant-libvirt --plugin-version=0.11.2
 echo ".. vagrant installed"
 echo
 set +e
@@ -82,17 +75,17 @@ if [[ -z "$G" ]]; then
   echo ".. You might need to log out and log in to start using netlab with libvirt"
   echo
 fi
-echo "Create vagrant-libvirt virtual network"
-set +e
-NET_LIST=$($SUDO virsh net-list --all|grep vagrant-libvirt)
-if [[ -n "$NET_LIST" ]]; then
-  echo ".. removing existing vagrant-libvirt network"
-  $SUDO virsh net-destroy vagrant-libvirt
-  $SUDO virsh net-undefine vagrant-libvirt
-fi
-SCRIPT_DIR=$(dirname "${BASH_SOURCE[0]}")
-set -e
-$SUDO virsh net-define "$SCRIPT_DIR/../templates/provider/libvirt/vagrant-libvirt.xml"
-echo ".. vagrant-libvirt network created"
-$SUDO virsh net-start vagrant-libvirt
-$SUDO virsh net-autostart vagrant-libvirt
+#echo "Create vagrant-libvirt virtual network"
+#set +e
+#NET_LIST=$($SUDO virsh net-list --all|grep vagrant-libvirt)
+#if [[ -n "$NET_LIST" ]]; then
+#  echo ".. removing existing vagrant-libvirt network"
+#  $SUDO virsh net-destroy vagrant-libvirt
+#  $SUDO virsh net-undefine vagrant-libvirt
+#fi
+#SCRIPT_DIR=$(dirname "${BASH_SOURCE[0]}")
+#set -e
+#$SUDO virsh net-define "$SCRIPT_DIR/../templates/provider/libvirt/vagrant-libvirt.xml"
+#echo ".. vagrant-libvirt network created"
+#$SUDO virsh net-start vagrant-libvirt
+#$SUDO virsh net-autostart vagrant-libvirt
